.TH PSGRAPH 1.1 "7 November 1990" DECWRL
.SH NAME
psgraph \- generate a graph in PostScript
.SH SYNOPSIS
.B psgraph
[ option ] [ files ]...
.SH DESCRIPTION
.I Psgraph
operates in a manner similar to
.I graph(1),
in that it reads pairs of numbers from its standard input
and produces a graph.
If files are given as arguments, each file is processed to produce a separate 
graph.
The output language is a complete 
.I PostScript 
document, suitable for inclusion in a Scribe or TeX document, or, with the 
.B \-P
flag, printing directly on a PostScript printer.
Several differences exist between
.I graph
and
.IR psgraph ,
the most significant of which is that
.I psgraph
can take all of its parameters
(except 
.BR \-P )
from its input instead of requiring that
everything except data be on the command line.
.SH OPTIONS
.PP
All the options of
.I graph
are supported, except
.BR \-s .
The meaning of the
.BR \-h ,
.BR \-w ,
and
.B \-c
options have changed.
A few new options have been added as well.
.TP
.B  \-h
Next argument is the height of the graph in inches.
.TP
.B  \-w
Similarly for width.
.TP
.B \-c
Center the X axis of the graph on the page width specified by the next argument
(in inches). 
.TP
.B \-P
Produce an output file with a 
.B showpage
command at the end of each graph, and a 
.B translate
command that assures that the entire graph will appear on the output page.
.TP
.B \-p
Use the file named by the next argument as the PostScript prologue, instead of 
the standard one. 
This allows you to define new line styles and marker shapes.
.SH INPUT
.PP
Input is line oriented; lines beginning with a pound sign are ignored.
Each line either specifies a data point (one or two numbers followed by an
optional label) or a command.
Lines are parsed into fields separated by white space.
A string of text enclosed in quotation marks or apostrophes is considered a
single field.
Command lines consist of a keyword followed by zero or more modifiers.
Some commands affect the entire graph (such as one to specify the grid type)
and so can be placed anywhere in the input (though placing them at the top
is conventional).
Other commands only affect the data that follows them (such as the one to
modify the line or marker type).
.PP
Data points in the input are interpreted as in
.I graph(1).
If the 
.B \-a
option is specified, only one number is expected, otherwise two.
If a label (second or third field) is given, that label is printed on the
graph at the location of the point given on the same line.
.PP
The following commands are recognized:
.sp
.IP CMD 1i
DESCRIPTION
.IP break 1i
Break the plot at this point in the input.
.IP "include F" 1i
Switch to reading input from the file
.IR F .
Yes, recursive 
.IR include s
are handled properly.
.IP "line S" 1i
Connect subsequent points with a line of type S,
where S is
.IR solid ,
.IR dotdashed ,
.IR shortdashed ,
.IR longdashed , 
.IR dotted , 
or
.IR off ,
with or without quotes.  If S is omitted, splines are turned
off (if on) and the line type remains unchanged.
.IP "color C" 1i
Set the color to be used in subsequent lines to be the one named "C",
where C is
.IR red ,
.IR blue ,
.IR magenta ,
.IR green ,
.IR black ,
.IR cyan ,
.IR yellow ,
.IR gray ,
.IR orange ,
or
.IR violet ,
with or without quotes. If C is omitted, the color is set to black.
.IP "linewidth D" 1i
Sets the width for subsequent graphed lines. The argument is a floating-point
number, and is the number of points of line width. There are 72 points to an
inch. The default line width is 0.6 points.
.IP "transparent" 1i
Normally, the label associated with a point is painted with a white background,
obliterating any text or graphics behind it. 
This command causes the text to be painted without first painting a white
background box.
.IP "notransparent" 1i
When painting a point label, first paint a white background box behind it.
.IP "label" 1i
Place numeric labels on the end of each grid line.
.IP "nolabel" 1i
Do not place numeric labels on grid lines.
.IP "tick N [M]" 1i
Make grid tick marks (for
.BR "grid tick" )
N inches long.
A second numeric parameter M states the length of secondary tick marks \-
those generated by the 
.B "x tick N"
command.
.IP datatick 1i
Place ticks only where there is a data point.
.IP datalabel 1i
Label every datatick.
.ne 3
.IP "grid N" 1i
Draw grid type N, 0=no grid, 1=frame grid, 2=frame grid with tick marks,
3=mesh grid, 4=frame grid with only left and bottom axes, 5=grid 4 with
tick marks.
Gets applied to both dimensions. Also, the more descriptive
.IR none , 
.IR open ,
.IR tick ,
.IR full ,
.IR halfopen ,
or
.I halftick
may be used.
For historical reasons, grid type 3, 
.IR full ,
is the default. 
.ne 2
.IP rangeframe 1i
For all grid types except
.I none
and
.IR full,
only draw the axis line over the range of the data on that axis.
Ticks or scales, if any, are drawn on the entire axis.
.IP "tickgray f"
Draw the ticks in f percent gray; default is 100% (full black).
.IP "axisgray f"
Draw the ticks in f percent gray; default is 100% (full black).
.ne 3
.IP "clip N"
Data points and connecting lines are clipped to the grid, to some
tolerance. The default tolerance is 0.05; that is, if the data value lies
up to 5% outside the axis limits, it will be plotted. The 
.B clip
verb is used to change the value of this tolerance. 
Log scale users should be warned that the calculation is done in ``linear
space'' and will often not behave as expected.
.IP "font f"
Switch to font 
.I f
for subsequent marker label text. 
Fonts are named as for 
.IR enscript(1) ,
e.g., ``Times-Roman10''.
.IP "width N" 1i
Make the grid N (float) inches wide.
Overridden by the
.B "size N"
command.
.IP "height N" 1i
Make the grid N (float) inches high.
Overridden by the
.B "size N"
command.
.IP "center N" 1i
Center the X axis on an N (float) inch wide line.
.IP "spline" 1i
Subsequent data is plotted as splines.  Causes
a break.
Markers, if turned on, still appear at the actual data point.
.IP "marker m" 1i
Plot subsequent points with marker type 
.IR m .
If 
.I m
is 
.IR none ,
.IR off ,
or omitted, markers are turned off.
Otherwise, valid values for 
.I m
are
.IR square , 
.IR diamond , 
.IR triangle , 
.IR up , 
.IR down , 
.IR right ,
.IR left , 
.IR circle , 
.IR x , 
.IR plus ,
.IR cross , 
.IR circle-x , 
.IR circle-plus ,
.IR filledsquare ,
.IR filleddiamond ,
.IR filledtriangle ,
and 
.IR filledcircle .
.IP "markerscale s" 1i
Scale the markers (if they are turned on) by the floating point scale value
.IR s .
The default value for s is 1.0.
.IP "markergray f" 1i
Draw the markers in f percent gray; default is 100% (full black). 
Any subsequent \fBcolor\fR command overrides this setting, and this
overrides only the marker color if a \fBcolor\fR command has been issued.
.IP "title \fIfoo\fR" 1i
Use 
.I foo
as the title for this plot (displayed at the top).
.IP "titlefont \fIbar\fR" 1i
Display the title in font
.IR bar .
Fonts are named as for
.IR enscript(1) ,
e.g., ``Times-Roman10''.
.IP "x args" 1i
Modify x data processing (see below)
.IP "y args" 1i
Modify y data processing (see below)
.LP
The arguments for the 
.B x 
and 
.B y 
commands can all be placed on a
single line, or separate 
.B x
and 
.B y
lines can be used for each.
.IP "log" 1i
Plot this dimension on a logarithmic scale.
.IP "min N" 1i
Minimum on the grid is N (float)
.IP "max N" 1i
Maximum on the grid is N (float)
.IP "step N" 1i
Distance between grid lines and numeric labels is N (float). No effect for
log scales.
.IP "tick N" 1i
Distance between secondary ticks is N (float). No effect for log scales.
.IP "intervals N" 1i
Number of intervals in a decade is N (integer). No effect for linear scales.
.IP datatick 1i
Place ticks only where there is a data point.
.IP datalabel 1i
Label every datatick.
.IP "offset N" 1i
Distance between border of page and minimum edge of the grid in this
dimension is N (float) inches.
.ne 2
.IP rangeframe 1i
For all grid types except
.I none
and
.IR full,
only draw the axis line over the range of the data on that axis.
Ticks or labels, if any, are drawn on the entire axis.
.IP "tickgray f"
Draw the ticks in f percent gray; default is 100% (full black).
.IP "axisgray f"
Draw the ticks in f percent gray; default is 100% (full black).
.IP "label \fIfoo\fR" 1i
Use
.I foo
as the label for this axis.
.IP "size N" 1i
Make the grid this big (N is in inches).
Overrides the 
.B width
and
.B height
commands.
.IP "grid N" 1i
Specify the grid type in this dimension (as above).
.IP "font \fIfoo\fR" 1i
Draw the axis labels in font
.IR foo .
Fonts are named as for 
.IR enscript(1) ,
e.g., ``Times-Roman10''.
.SH FILES
/usr/local/lib/ps/psgraph.pro \- standard prologue
.SH SUGGESTIONS
Many of the options are provided to minimize 
.BR chartjunk ,
that is, to maximize the data to ink ratio. 
To this end, the grid types 
.I halfopen
and
.I halftick
are the most useful.
.PP
.I Rangeframe
and 
.I datatick
were designed to make a standard bivariate scatterplot more useful.
.PP
All the graphical elements of the frame can be removed, either by careful
selection of the grid type, the various gray settings, or 
.IR nolabel .
.SH "SEE ALSO"
enscript(1), graph(1), psgsimp(1)
.PP
Tufte, Edward, R. 
.I "The Visual Display of Quantitative Information."
Graphics Press, P.O. Box 430, Cheshire, CT. 1983.
.PP
Tufte, Edward, R. 
.I "Envisioning Information."
Graphics Press, P.O. Box 430, Cheshire, CT. 1990.
.SH BUGS
A superset of the bugs listed in
.I graph(1).
.PP
There are almost certainly too many options.
